﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WarmlyLocomotive
{
    internal class MapWithSetLocomotivesGeneric<T, U>
        where T : class, IDrawningObject
        where U : AbstractMap
    {
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Размер занимаемого объектом места (ширина)
        /// </summary>
        private readonly int _placeSizeWidth = 210;
        /// <summary>
        /// Размер занимаемого объектом места (высота)
        /// </summary>
        private readonly int _placeSizeHeight = 120;
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly SetLocomotivesGeneric<T> _setLocomotives;
        /// <summary>
        /// Карта
        /// </summary>
        private readonly U _map;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="picWidth"></param>
        /// <param name="picHeight"></param>
        /// <param name="map"></param>
        public MapWithSetLocomotivesGeneric(int picWidth, int picHeight, U map)
        {
            int width = picWidth / (_placeSizeWidth + 30);
            int height = picHeight / _placeSizeHeight;
            _setLocomotives = new SetLocomotivesGeneric<T>(width * height);
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _map = map;
        }
        /// <summary>
        /// Перегрузка оператора сложения
        /// </summary>
        /// <param name="map"></param>
        /// <param name="locomotive"></param>
        /// <returns></returns>
        public static int operator +(MapWithSetLocomotivesGeneric<T, U> map, T locomotive)
        {
            return map._setLocomotives.Insert(locomotive);
        }
        /// <summary>
        /// Перегрузка оператора вычитания
        /// </summary>
        /// <param name="map"></param>
        /// <param name="position"></param>
        /// <returns></returns>
        public static T operator -(MapWithSetLocomotivesGeneric<T, U> map, int position)
        {
            return map._setLocomotives.Remove(position);
        }
        /// <summary>
        /// Вывод всего набора объектов
        /// </summary>
        /// <returns></returns>
        public Bitmap ShowSet()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawLocomotives(gr);
            return bmp;
        }
        /// <summary>
        /// Просмотр объекта на карте
        /// </summary>
        /// <returns></returns>
        public Bitmap ShowOnMap()
        {
            Shaking();
            foreach (var locomotive in _setLocomotives.GetLocomotives())
            {
                return _map.CreateMap(_pictureWidth, _pictureHeight, locomotive);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// <summary>
        /// Перемещение объекта по крате
        /// </summary>
        /// <param name="direction"></param>
        /// <returns></returns>
        public Bitmap MoveObject(Direction direction)
        {
            if (_map != null)
            {
                return _map.MoveObject(direction);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// <summary>
        /// Получение данных в виде строки
        /// </summary>
        /// <param name="sep"></param>
        /// <returns></returns>
        public string GetData(char separatorType, char separatorData)
        {
            string data = $"{_map.GetType().Name}{separatorType}";
            foreach (var locomotive in _setLocomotives.GetLocomotives())
            {
                data += $"{locomotive.GetInfo()}{separatorData}";
            }
            return data;
        }
        /// <summary>
        /// Загрузка списка из массива строк
        /// </summary>
        /// <param name="records"></param>
        public void LoadData(string[] records)
        {
            foreach (var rec in records)
            {
                _setLocomotives.Insert(DrawningObjectLocomotive.Create(rec) as T);
            }
        }
        /// <summary>
        /// "Взбалтываем" набор, чтобы все элементы оказались в начале
        /// </summary>
        private void Shaking()
        {
            int j = _setLocomotives.Count - 1;
            for (int i = 0; i < _setLocomotives.Count; i++)
            {
                if (_setLocomotives[i] == null)
                {
                    for (; j > i; j--)
                    {
                        var locomotive = _setLocomotives[j];
                        if (locomotive != null)
                        {
                            _setLocomotives.Insert(locomotive, i);
                            _setLocomotives.Remove(j);
                            break;
                        }
                    }
                    if (j <= i)
                    {
                        return;
                    }
                }
            }
        }
        /// <summary>
        /// Метод отрисовки фона
        /// </summary>
        /// <param name="g"></param>
        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            Pen penTwo = new(Color.Black, 2);
            Brush brushLightGray = new SolidBrush(Color.LightGray);
            Brush brushBrown = new SolidBrush(Color.Brown);
            g.FillRectangle(brushLightGray, 0, 0, _pictureWidth, _pictureHeight);
            int placesInRow = _pictureWidth / (_placeSizeWidth + 30);
            for (int i = 0; i < placesInRow; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
                {//линия рамзетки места
                    if (j != 0)
                    {
                        int xForRail = i * (_placeSizeWidth + 30);
                        g.DrawLine(penTwo, xForRail, j * _placeSizeHeight, xForRail + _placeSizeWidth - 20, j * _placeSizeHeight);
                        g.DrawLine(penTwo, xForRail, j * _placeSizeHeight - 20, xForRail + _placeSizeWidth - 20, j * _placeSizeHeight - 20);
                        for (int k = 0; k < (_placeSizeWidth - 20) / 15; ++k)
                        {
                            g.FillRectangle(brushBrown, xForRail, j * _placeSizeHeight - 20, 5, 20);
                            xForRail += 15;
                        }
                    }
                    g.DrawLine(pen, i * (_placeSizeWidth + 30), j * _placeSizeHeight + 10, i * (_placeSizeWidth + 30) + _placeSizeWidth / 2, j * _placeSizeHeight + 10);
                }
                g.DrawLine(pen, i * (_placeSizeWidth + 30), 10, i * (_placeSizeWidth + 30), (_pictureHeight / _placeSizeHeight) * _placeSizeHeight + 10);
            }
        }
        /// <summary>
        /// Метод прорисовки объектов
        /// </summary>
        /// <param name="g"></param>
        private void DrawLocomotives(Graphics g)
        {
            int xForLocomotive = 2;
            int yForLocomotive = 10;
            int countInRow = 0;
            foreach (var locomotive in _setLocomotives.GetLocomotives())
            {
                if (countInRow >= _pictureWidth / (_placeSizeWidth + 30))
                {
                    xForLocomotive = 2;
                    yForLocomotive += _placeSizeHeight;
                    countInRow = 0;
                }
                if (locomotive != null)
                {
                    locomotive.SetObject(xForLocomotive, yForLocomotive, _pictureWidth, _pictureHeight);
                    locomotive.DrawningObject(g);
                }
                xForLocomotive += _placeSizeWidth + 30;
                countInRow++;
            }
        }
    }
}
